/*
 * Query - Send query to domain name server
 *
 * Copyright (C) Meilof Veeningen, 2002
 */

#include <config.h>

#include "query.h"

#include <syssocket.h>
#include <domainfn.h>
#include <classes.h>

#include <include/unistring.h>

query_info::query_info() {
    errmsg = "";
    disconnected = 1;
}

query_info::~query_info() {
}

int query_info::connect(_addr *_server, _transport trans) {
  if (disconnected == 0) disconnect();
    errmsg = "";
	initsocklib();
	memcpy(&server, _server, sizeof(_addr));
	transport = trans;
    _addr addr;

	switch (transport) {
		case PROTO_UDP:
            if (is_ipv4(&server)) getaddress(&addr, "0.0.0.0", 0);
#ifdef HAVE_IPV6
            if (is_ipv6(&server)) getaddress(&addr, "::", 0);
#endif
			sockid = udpcreateserver(&addr);
			if (sockid < 0){
        errmsg = "Unable to create UDP socket";
        return sockid;
      }
			break;
		case PROTO_TCP:
			sockid = tcpopen(&server);
			if (sockid < 0) {
        errmsg = "Unable to TCP connect";
        return sockid;
      }
			break;
	}
  disconnected = 0;
	return 0;
}

int query_info::disconnect() {
    errmsg = "";
	if (sockid > 0) {
		if (transport == PROTO_UDP)
			udpclose(sockid);
		else
			tcpclose(sockid);
	}
	closesocklib();
  disconnected = 1;
	return 0;
}

int query_info::nsquery(DnsMessage *_query, DnsMessage** _answer, int _timeout) {
  query = _query;
  answer = _answer;
  curtimeout = _timeout;

  retcode = -1;
  doquery();
  return retcode;
}

void *query_info::doquery() {
  int timeout;
	AgBuffer message;
	int ret, left, len;
	char *buff = NULL; char lenbuff[2];
	_addr sender;

    errmsg = "";

    if (curtimeout) timeout = curtimeout; else timeout = this->timeout;

  	watchset_init();

  	/* compile message */
  	ret = query->compile(&message, (transport == PROTO_UDP) ? UDP_MSG_SIZE : -1);
  	if (ret < 0) { errmsg = "Could not compile query"; goto _error; }


  	/* send message and wait */
  	if (transport == PROTO_UDP) {
  		ret = udpsend(sockid, &server, message.buff, message.bufflen);
  		if (ret < 0) { errmsg = "Could not UDP send query"; goto _error; }
  		buff = (char *)malloc(UDP_MSG_SIZE);
  		watchset_add(sockid);
  		if (watchset_waitread(timeout) <= 0) { errmsg = "No answer"; free(buff); goto _error; }
  		if (!watchset_isdata(sockid)) { errmsg = "No answer"; free(buff); goto _error; }
  		len = udpread(sockid, buff, &sender);
  		if (len < 0) { errmsg = "Could not read answer"; free(buff); goto _error; }
  		if (!compare_host(&server, &sender)) { errmsg = "Answer from unexpected host"; free(buff); goto _error; }
  	} else {
  		left = message.bufflen;
  		lenbuff[0] = message.bufflen / 256;
  		lenbuff[1] = message.bufflen;

  		if (tcpsend(sockid, lenbuff, 2) != 2) { errmsg = "Could not send two length bytes"; goto _error; }
  		while (left > 0) {
  			ret = tcpsend(sockid, &message.buff[message.bufflen - left], left);
  			if (ret < 0) { errmsg = "Could not TCP send message"; goto _error; }
  			left -= ret;
  		}
  		watchset_add(sockid);
  		watchset_waitread(timeout);
  		if (!watchset_isdata(sockid)) { errmsg = "No answer"; goto _error; }
  		ret = tcpread(sockid, lenbuff, 2);
  		if (ret < 2) { errmsg = "Could not read answer"; goto _error; }
  		len = lenbuff[0] * 256 + lenbuff[1];
  		buff = (char *)malloc(len);
  		left = len;
  		while (left > 0) {
  			watchset_waitread(1);
  			if (!watchset_isdata(sockid)) { errmsg = "Could not read answer"; free(buff); goto _error; }
  			ret = tcpread(sockid, &buff[len - left], left);
  			if (ret < 0) { errmsg = "Could nor read answer"; free(buff); goto _error; }
  			left -= ret;
  		}
  	}

   if (!buff) { errmsg = "Internal error"; return (void *)-1; }

  	*answer = new DnsMessage();

  	if ((*answer)->loadFromData(buff, len) < 0) {
      errmsg = "Malformed answer";
  		delete answer;
      *answer = NULL;
      goto _error;
  		retcode = -1; return NULL;
  	}

    free(buff);
    retcode = 0;
    goto _aftererr;

_error:
    retcode = -1;

_aftererr:
    
	return NULL;
}

DnsMessage *make_query(_domain dname, u_short qtype) {
	DnsMessage *q = new DnsMessage();
	q->question.add(new question_item(dname, (u_short)qtype));
	return q;
}
