/*
    Posadis - A DNS Server
    The AutoGrowBuffer - A char buffer which automagically grows as you cat information to it
    Copyright (C) 2001  Meilof Veeningen <meilof@users.sourceforge.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "agb.h"
#include "include/unistring.h"

char agbpow2[8] = { 128, 64, 32, 16, 8, 4, 2, 1 };

const double agb_buffmargin = 1.25;


AgBuffer::AgBuffer() {
    unreg();
}

AgBuffer::~AgBuffer() {
    free(buff);
}

void AgBuffer::enlarge(int len) {
    _enlarge(len);
    bufflen += len;
}

void AgBuffer::cat(char *tbuff, int tbufflen) {
    if (tbufflen == 0) return;
    enlarge(tbufflen);
    memcpy(&buff[bufflen-tbufflen], tbuff, tbufflen);
}

void AgBuffer::catstring(char *str) {
	cat(str, strlen(str));
}

void AgBuffer::catint(int value) {
    _enlarge(4);
    buff[bufflen++] = (unsigned)value / 16777216;
    buff[bufflen++] = (unsigned)value / 65536;
    buff[bufflen++] = (unsigned)value / 256;
    buff[bufflen++] = (unsigned)value;
}

void AgBuffer::catshort(short value) {
    enlarge(2);
    modshort(bufflen - 2, value);
}

void AgBuffer::modshort(int pos, int value) {
    buff[pos] = (unsigned short)value / 256;
    buff[pos+1] = (unsigned short)value;
}

void AgBuffer::catchar(char value) {
    _enlarge(1);
    buff[bufflen++] = value;
}

void AgBuffer::setbit(int pos, int bit) {
    if (pos >= bufflen) {
        enlarge(pos - bufflen + 1);
        memset(&buff[pos], 0, bufflen - pos);
    }
    buff[pos] |= agbpow2[bit];
}

void AgBuffer::set4bit(int pos, int bitfrom, int size) {
    if (pos >= bufflen) {
        enlarge(pos - bufflen + 1);
        memset(&buff[pos], 0, bufflen - pos);
    }
    buff[pos] |= size * agbpow2[bitfrom + 3];

}

void AgBuffer::_enlarge(int len) {
	if (bufflen + len >= reallen) {
		reallen = (int) ( (double)(bufflen + len) * agb_buffmargin ) + 32;
		buff = (char *)realloc(buff, reallen);
	}
}

void AgBuffer::cutto(int len) {
	if (len < bufflen) {
		if ( (double) reallen / agb_buffmargin >= len ) {
			reallen = len;
			buff = (char *)realloc(buff, len);
		}
	}
	bufflen = len;
}

void AgBuffer::unreg() {
    buff = NULL; bufflen = 0; reallen = 0;
}
