/*
    Posadis - A DNS Server
    Class which holds a list of bits
    Copyright (C) 2001  Meilof Veeningen <meilof@users.sourceforge.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "bitlist.h"
#include "include/unistring.h"

#define BYTES_INDEX(ix)         (ix / 8)
#define MASK_INDEX(ix)          (ix % 8)

BitList::BitList(long nValues) {
        bytes = (char *)malloc((nValues + 7) / 8);
        memset(bytes, 0, (nValues + 7) / 8);
        nVals = nValues;
}

BitList::~BitList() {
        free(bytes);
}

bool BitList::isset(long ix) {
        if (ix >= nVals) return false;
        return (bytes[BYTES_INDEX(ix)] & (1 << MASK_INDEX(ix)));
}

int BitList::set(long ix) {
        if (ix >= nVals) return false;
        bytes[BYTES_INDEX(ix)] |= (1 << MASK_INDEX(ix));
        return 1;
}

int BitList::unset(long ix) {
        if (ix >= nVals) return false;
        bytes[BYTES_INDEX(ix)] &= ~(1 << MASK_INDEX(ix));
        return 1;
}

int BitList::toggle(long ix) {
        if  (ix >= nVals) return false;
        bytes[BYTES_INDEX(ix)] ^= (1 << MASK_INDEX(ix));
        return 1;
}

