/*
    Posadis - A DNS Server
    Implementation of the local DNS cache
    Copyright (C) 2001  Meilof Veeningen <meilof@users.sourceforge.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "include/unistring.h"
#include <time.h>

#include "cache.h"
#include "dnsconstants.h"
#include "posadis.h"
#include "rr.h"

_rr_data::_rr_data(int _RDLENGTH, char *_RDATA) {
	RDATA = _RDATA;
	RDLENGTH = _RDLENGTH;
}

_rr_data::~_rr_data() {
	free(RDATA);
}

_rr_data *_rr_data::clone() {
	return new _rr_data(RDLENGTH, (char *)memdup(RDATA, RDLENGTH));
}

rr_set::rr_set(int _type) { type = _type; init(); }
rr_set::~rr_set() { }

void rr_set::init() {
    expire = -2; ttl = 0; last = 0;
}

void rr_set::destroy() {
    items.destroy();
}

void rr_set::re_init() {
    destroy();
    init();
}

bool rr_set::add_rr(bool is_auth, int _TTL, int _RDLENGTH, char *_RDATA) {
	char *rdata;
	rdata = (char *)malloc(_RDLENGTH);
	memcpy(rdata, _RDATA, _RDLENGTH);
	items.add(new _rr_data(_RDLENGTH, rdata));
    if (is_auth) {
        if (expire == -2 || _TTL < ttl) { ttl = _TTL; expire = -1; }
    } else {
        if (expire == -2 || _TTL + curtime < expire) { expire = _TTL + curtime; ttl = -1; }
    }
    return true;
}

bool rr_set::add_rr(bool is_auth, complete_rr *rr) {
    return add_rr(is_auth, rr->TTL, rr->RDLENGTH, rr->RDATA);
}


rr_set *rr_set::clone() {
	rr_set *newset;
	int x;
	
	newset = new rr_set(type);
	newset->expire = expire;
	newset->ttl = ttl;
	newset->last = last;
	newset->type = type;
	
	for (x = 0; x < items.count; x++) newset->items.add(items[x]->clone());
	
	return newset;
}

CacheItem::CacheItem(char *_name) {
	name = strdup(_name);
}

CacheItem::~CacheItem() {
    free(name);
}

rr_set *CacheItem::get_rrset(u_short type) {
	int t;
	
	for (t = 0; t < rr_sets.count; t++)
		if (rr_sets[t]->type == type) return rr_sets[t];
		
	return NULL;
}

bool CacheItem::remove_rrset(u_short type) {
     rr_set *ptr = get_rrset(type);
     if (ptr)
         return rr_sets.removeitem(ptr);
     else
         return false;
}

rr_set *CacheItem::write_rrset(u_short type) {
	rr_set *set;
	
	set = get_rrset(type);
	if (set == NULL) {
		set = new rr_set(type);
		rr_sets.add(set);
	}
	
	return set;
}
		
bool CacheItem::add_msg_rr(complete_rr *rr, bool is_auth) {
	rr_set *set;
	set = write_rrset(rr->TYPE);
	if (!set) return false;
	return set->add_rr(is_auth, rr->TTL, rr->RDLENGTH, rr->RDATA);
}


bool CacheItem::remove_msg_rr(complete_rr *rr) {
	rr_set *set;
	int x;
	
	set = get_rrset(rr->TYPE);
	if (!set) return false;
	
	for (x = 0; x < set->items.count; x++) {
		if (set->items[x]->RDLENGTH == rr->RDLENGTH &&
			memcmp(set->items[x]->RDATA, rr->RDATA, rr->RDLENGTH) == 0) {
			set->items.removeindex(x);
			goto found;
        }
	}
	return false;
found:	
	if (set->items.count == 0) rr_sets.removeitem(set);
	return true;
}

bool CacheItem::remove_subitem(char *name) {
    int x;

    for (x = 0; x < subitems.count; x++) {
        if (stricmp(subitems[x]->name, name) == 0) {
            subitems.removeindex(x);
            break;
        }
    }

    return true;
}

CacheItem *CacheItem::clone() {
	CacheItem *citem;
	int x;
	
	citem = new CacheItem(name);
	for (x = 0; x < rr_sets.count; x++) citem->rr_sets.add(rr_sets[x]->clone());
	for (x = 0; x < subitems.count; x++) citem->subitems.add(subitems[x]->clone());

	return citem;	
}
