/*
    Posadis - A DNS Server
    Headers for the local DNS cache
    Copyright (C) 2001  Meilof Veeningen <meilof@users.sourceforge.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef __CACHE_H
#define __CACHE_H

class rr_set;
class CacheItem;

#include <time.h>
#include "dnstypes.h"
#include "zones.h"
#include "classes.h"
#include "vector.h"

int cache_init();                           // initalizes the cache

class _rr_data {                            // contains actual RR data, but no domain name
  public:
	_rr_data(int RDLENGTH, char *RDATA);	// constructor
	~_rr_data();							// destructor
	
    int RDLENGTH;                           // number of bytes
    char *RDATA;                            // the data

    _rr_data *clone();						// clones the _rr_data
};

class rr_set {                              // a set of resources of a particular type
  public:
    rr_set(int _type);                      // constructor
    ~rr_set();                              // destructor

    void re_init();                         // reinitializes the rr_set so that the data can be replaced
    bool add_rr(bool is_auth, int TTL, int RDLENGTH, char *RDATA); // adds a rr to the rr_set (is_auth is to determine whether the TTL is fixed (authoritive), or a timeout (cache)
    bool add_rr(bool is_auth, complete_rr *rr);

    int expire;                             // the time at which this data will expire (or -1 if we're authoritive, or -2 if not initialized)
    int ttl;                                // the time to live (or -1 if we're not authoritive)
    int last;                               // the last member issued (for item cycling)
    int type;                               // the type of data the RRset describes
    Vector<_rr_data *>items;				// data which describes the resource

    rr_set *clone();						// clones the rr set
  private:
    void init(); void destroy();            // called by [con|de]structors and re_init()
};

class CacheItem {                           // class for an item in the cache
    public:
    CacheItem(char *name);                  // constructor
    ~CacheItem();                           // destructor

    char *name;                             // the name, relative to the parent
    										// for zone root items, this is empty

    Vector<rr_set*> rr_sets;				// the properties of this item
    rr_set *get_rrset(u_short type);		// returns the rrset of a certain type
    rr_set *write_rrset(u_short type);		// returns the rrset of a cetrain type, creating it if nessecary
    bool remove_rrset(u_short type);        // removes a complete rr set
    bool add_msg_rr(complete_rr *rr, bool auth); // adds RR from DnsMessage to this cache item
    bool remove_msg_rr(complete_rr *rr);	// removes RR from the item (useful for IXFR, UPDATE)

    CacheItem *clone();						// clones the complete CacheItem

    Vector<CacheItem*> subitems;			// the subitems of the domain
    bool remove_subitem(char *name);        // removes subitem
};

#endif /* ifndef __CACHE_H */

