/*
    Posadis - A DNS Server
    DNS foundation classes
    Copyright (C) 2001  Meilof Veeningen <meilof@users.sourceforge.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef __CLASSES_H
#define __CLASSES_H

#ifndef NULL
#define NULL 0
#endif

class question_item;
class complete_rr;
class DnsMessage;

#include "dnstypes.h"
#include "agb.h"
#include "domainfn.h"
#include "vector.h"
#ifndef _COMPILE_AS_LIBRARY
#include "cache.h"
#endif /* _COMPILE_AS_LIBRARY */
typedef Vector<complete_rr*> _dm_section;

class question_item {                       // contains an entry of the question section of a dns message
    public:
    question_item();                        // constructor
    question_item(_domain dom, u_short qtype);
    ~question_item();                       // destructor
    _domain QNAME;                          // the domain name queried
    u_short QTYPE;                          // the question type
    u_short QCLASS;                         // the question class
};

class complete_rr {                         // contains all RR data
    public:
    complete_rr();                          // constructor
    complete_rr(_domain NAME, u_short TYPE, u_short CLASS, u_int TTL, u_short RDLENGTH, char *RDATA);
    ~complete_rr();                         // destructor
    _domain NAME;                           // the domain name to which the RR applies
    u_short TYPE;                           // the RR type
    u_short CLASS;                          // the class (CLASS_IN)
    u_int TTL;                              // the time to live for the rrset
    u_short RDLENGTH;                       // number of bytes
    char *RDATA;                            // the data
};

enum _secttype {
    S_ANSWER,
    S_AUTHORITY,
    S_ADDITIONAL,
};

class DnsMessage {
  public:
    DnsMessage();                           // constructor
#ifndef _COMPILE_AS_LIBRARY
    DnsMessage(DnsMessage *q);				// constructor making an answer to *q
#endif
    ~DnsMessage();                          // destructor

    u_short ID;                             // unique identifier; given by client and copied by server
    bit QR;                                 // is it a query (false) or a response (true)
    bit4 OPCODE;                            // the operation code (an OPCODE_ constant)
    bit AA;                                 // authoritive answer?
    bit TC;                                 // specifies whether the answer was truncenated
    bit RD;                                 // recursion demanded?
    bit RA;                                 // recursion available?
    u_short Z;                              // three reserved bits
    bit4 RCODE;                             // answer type (a RCODE_ constant)

    Vector<question_item*> question;		// the questions section
    _dm_section answer;						// the answer section
    _dm_section authority;					// the answer section
    _dm_section additional;					// the answer section


    void init();                            // initializes all values
    void clean();                           // cleans all allocated memory
    void reinit();                          // reinitializes the class

    int readsection(_dm_section& section, int nitems, char *data, int &pos, int dlen);
    int loadFromData(char *data, int dlen);

    int write_rr(complete_rr *rr, AgBuffer *buff);
    int compile(AgBuffer *buff, int maxlen);

	_dm_section& getsection(_secttype type);

#ifndef _COMPILE_AS_LIBRARY
    int answer_init(DnsMessage *question, bool copy_request = true);
    int is_answer_on(DnsMessage *question);

    bool rrset_exists(_domain dname, u_short rrtype, _secttype section);

    int do_additional_section_processing();
    int do_additional_section_processing_for(_secttype section = S_ANSWER);

	int add_rrset(_domain dname, rr_set* rrset,
		bool duplicheck = true, _secttype section = S_ANSWER);
	int add_rrset_qtype(_domain dname, CacheItem *item, int qtype,
						 bool duplicheck = true, _secttype section = S_ANSWER);
#endif /* _COMPILE_AS_LIBRARY */
};

#ifndef _COMPILE_AS_LIBRARY

/*! Find out what question in a standard query needs to be answered. If there
    are no entries in the answer section, this obviously is the QNAME of the
    question. But, if the answer section already contains CNAMEs, this is the
    domain name the last CNAME in the DNS message points to.

    Note that this function returns a direct pointer to an area in the
    DnsMessage. Thus, this data should NOT be edited!
*/
_domain get_current_qname(DnsMessage *a);

/*! Check the message and see if we've already reached the maximum cname level
    (true) or the query can be restarted (false) */
bool reached_max_cname_level(DnsMessage *a);

#endif /* ifndef _COMPILE_AS_LIBRARY */

int write_compressed_domain(_domain domain, AgBuffer *buff);

#endif /* ifndef __CLASSES_H */

