/*
    Posadis - A DNS Server
    Simple domain-manipulation functions
    Copyright (C) 2001  Meilof Veeningen <meilof@users.sourceforge.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef __DOMAINFN_H
#define __DOMAINFN_H

#include <config.h>

#ifndef NULL
#define NULL 0
#endif

#include "dnstypes.h"

#define DOM_MAX         255              /* max length of domain name */
#define DOMLABEL_MAX    63               /* max length of dom label value */
#define DOMLABEL_RMAX   DOMLABEL_MAX + 2 /* max length of domlabel() return */

void *memdup(void *src, int len);

inline int domlabellen(char *label); /* returns the length, in bytes, of the dns label pointed to by label */

int comprdomlen(_domain domain, int maxbufflen, char *compr, int comprlen); // the function domlen calls when compressed data is given
int domlen(_domain domain, int maxbufflen = 0, char *compr = NULL, int comprlen = 0);
 // returns the length in bytes of the domain name (compr is compressed data)
 // if the data is not compressed, it just returns the length.
 // if it's compressed:
 //  - if compr and comprlen are given, the length of the complete domain name is returned
 //  - if they are not given, the length it takes up in the buffer is returned

int domlabelcount(_domain domain);          // returns the number of labels of the domain name
int domlabel(char *target, _domain domain, int index); // writes the (index)th label to target

int domrelabel(_domain target, char *src); // convers domlabel() output to domain name

int domfrom(char *target, _domain domain, int index); // writes all portions of the domain name from the (index)th label
int domto(char *target, _domain domain, int index); // write all portions of the domain name to (non-inclusive) the (index)th label

bool domcat(_domain target, _domain src); // cats src to target
bool domcpy(_domain target, _domain src); // copy src to target

bool domisparent(_domain parent, _domain child); // returns true if 'child' is a child of 'parent'

int domtostring(char *target, _domain src); // converts a domain to a human-readable string (with dots)

_domain domdup(_domain domain, int maxbufflen = 255, char *compr = NULL, int comprlen = 0); // duplicates a domain (compr is data for compression; maxbufflen is the maximum size of the buffer the domain resides in)

bool domcmp(_domain dom1, _domain dom2); // returns true if the domains are the same
bool advdomcmp(_domain dom1, int ix, char *compr, int comprlen); // returns true if the non-compressed d1 is the same as the (perhaps compressed) d2

int domlabelfromname(_domain res, char *src); // creates a binary label from a domlabel() return value; places a \0 after

#ifdef HAVE_BITSTRING

#define domlabeltype(val) (val & 192)
#define domlabelrest(val) (val & 63)

#define dlt_common 0
#define dlt_compr 192
#define dlt_edns0 64

#define dlt_bitstring 1

#endif

#define bitstr_count(x) ( (x == 0) ? 256 : x )
#define bitstr_val(x) ( (x == 256) ? 0 : x )
#define bitstr_len(x) ( (bitstr_count(x) + 7) / 8 )

#define bitset(val, x) ( (val)[(x) / 8] |= (char)(1 << (7 - ((x) % 8))) )
#define bitreset(val, x) ( (val)[(x) / 8] &= (char)~(1 << (7 - ((x) % 8))) )
#define bitsetval(val, x, y) ( (y) ? bitset(val, x) : bitreset(val, x) )
#define bitisset(val, x) ( ((val)[(x) / 8] & (1 << (7 - ((x) % 8)))) ? 1 : 0 )

void bitstringcpy(char *res, int resbit, char *src, int srcbit, int nbits);
int bitstringcmp(char *val1, int val1bit, char *val2, int val2bit, int nbits);


#endif /* ifndef __DOMAINFN_H */

