/*
    Posadis - A DNS Server
    Function which finds configuration directory locations.
    Copyright (C) 2001  Meilof Veeningen <meilof@users.sourceforge.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <include/unifile.h>
#include <include/unistring.h>

#ifdef _WIN32
#include <windows.h>
#include <winreg.h>
#endif

#ifdef _WIN32
char pathdelim[2] = "\\";
#else
char pathdelim[2] = "/";
#endif

bool is_posadis_dir(char *testname, char *dir) {
    /* this is determined by the existence of a file called <testname> */
    char fname[PATH_MAX];
    int file;

    if (strlen(dir) + strlen(testname) + 1 > sizeof(fname)) return false;

    strcpy(fname, dir);
    strcat(fname, testname);

    file = open(fname, O_RDONLY);
    if (file >= 0) { close(file); return true; }
    return false;
}

inline void str_to_upper(char *src, char *res) {
    /* i believe neither std-c nor std-c++ has a function for this */
    while (*src) *(res++) = toupper(*(src++));
    *res = 0;
}

int get_config_dir(char *dirtype, char result[PATH_MAX], char *testfile) {
    char buff[128];
    char *val;
#ifdef _WIN32
    HKEY hkey;
    DWORD valuetype;
    DWORD buffsize = PATH_MAX - 1; /* one extra for path delimiter on end */
#endif

    if (testfile == NULL) testfile = dirtype;

    result[0] = '\0';
    /* try to determine the correct configuration directory in a few steps */
#ifdef _WIN32
    /* try to retrieve the
 [HKEY_LOCAL_MACHINE\Software\Posadis]-><type>-directory key */
    valuetype = 16;
    strcpy(buff, dirtype); strcat(buff, "-directory");
    if ((RegOpenKeyEx(HKEY_LOCAL_MACHINE, "Software", 0, KEY_READ, &hkey) ==
 ERROR_SUCCESS) && \
        (RegOpenKeyEx(hkey, "Posadis", 0, KEY_READ, &hkey) == ERROR_SUCCESS)
 && \
        (RegQueryValueEx(hkey, buff, NULL, &valuetype, (BYTE*)result,
 &buffsize) == ERROR_SUCCESS) && \
        valuetype == REG_SZ) {
            if (result[strlen(result)-1] != pathdelim[0]) strcat(result,
 pathdelim);
            if (is_posadis_dir(testfile, result)) return 0;
    }
#endif
    /* see if there is an environment variable called POSADIS_<type>_DIR */
    strcpy(buff, "POSADIS_");
    str_to_upper(dirtype, &buff[strlen(buff)]);
    strcat(buff, "_DIR");
    val = getenv(buff);
    if (val != NULL && strlen(val) + 1 < PATH_MAX) {
        strcpy(result, val);
        if (result[strlen(result)-1] != pathdelim[0]) strcat(result,
 pathdelim);
        if (is_posadis_dir(testfile, result)) return 0;
        return 0;
    }
    /* see if we can find it in ../dirtype (if the executable is placed in the
      'src' directory of posadis) */
    strcpy(result, "..");
    strcat(result, pathdelim);
    strcat(result, dirtype);
    strcat(result, pathdelim);
    if (is_posadis_dir(testfile, result)) return 0;
#ifdef _WIN32
    /* try some win32 default values */
    strcpy(result, "\\dns\\");
    strcat(result, dirtype);
    strcat(result, pathdelim);
    if (is_posadis_dir(testfile, result)) return 0;
#else
    /* try some unix defaults:
        /etc/posadis,
        /var/posadis,
        /opt/posadis,
        /usr/share/posadis,
        /usr/local/posadis
    */
    strcpy(result, "/etc/posadis/");
    if (is_posadis_dir(testfile, result)) return 0;
    strcat(result, dirtype);
    strcat(result, pathdelim);
    if (is_posadis_dir(testfile, result)) return 0;

    strcpy(result, "/var/posadis/");
    if (is_posadis_dir(testfile, result)) return 0;
    strcat(result, dirtype);
    strcat(result, pathdelim);
    if (is_posadis_dir(testfile, result)) return 0;

    strcpy(result, "/opt/posadis/");
    if (is_posadis_dir(testfile, result)) return 0;
    strcat(result, dirtype);
    strcat(result, pathdelim);
    if (is_posadis_dir(testfile, result)) return 0;

    strcpy(result, "/usr/share/posadis/");
    if (is_posadis_dir(testfile, result)) return 0;
    strcat(result, dirtype);
    strcat(result, pathdelim);
    if (is_posadis_dir(testfile, result)) return 0;

    strcpy(result, "/usr/local/posadis/");
    if (is_posadis_dir(testfile, result)) return 0;
    strcat(result, dirtype);
    strcat(result, pathdelim);
    if (is_posadis_dir(testfile, result)) return 0;
#endif

    return -1;
}

