/*
    Posadis - A DNS Server
    Hint Zones
    Copyright (C) 2001  Meilof Veeningen <meilof@users.sourceforge.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef __HINT_H
#define __HINT_H

#include "zones.h"

/* hint zone group */

class HintGroup: public ZoneGroup { public: HintGroup(); hint_configuration_data hint_config; };

/*!
  The HintZone is the zone for a non-authoritive hint zone. All a hint zone
  does is sending back a list of nameservers that might be of use to the user.

  An option of the HintZone will be to regularly check in at some nameserver
  to get a more recent list of authoritive nameservers for the zone.

  This class is currently a subclass of NotauthZone, the base class for all
  non-authoritive zones.
*/

class HintZone : public NotauthZone {
  public:
    HintZone(_domain domain);                           // public constructor
    virtual ~HintZone();

    virtual void offer_glue(CacheItem *item);           // we're getting some glue!
                                                        // if this is the case, it surpasses hard-coded defaults
    virtual void start_updating();                      // overrided to load (if configured) from some DNS server using standard queries, or from a "master file"
    virtual void set_hardcoded();                       // sets hardcoded nameservers

    hint_configuration_data hint_config;                // some settings
    enum { SRC_NONE, SRC_GLUE, SRC_AUTH } datasrc;

    virtual _thread_answer do_lookup_thread(AnsweringThread*& thr, stdquery_info *info, _domain QNAME);
};

class HintUpdateThread : public MainThread {
  public:
    HintUpdateThread(HintZone *_zone, Vector<_addr *> *_servers);
    ~HintUpdateThread();
    virtual int execute();
    int add_to_cache(_dm_section *sec);

    DnsMessage *q;
    enum { STATE_INIT, STATE_RECV, STATE_SEND } state;
    int cur;
    Thread *child;
    Vector<_addr *> *servers;
    HintZone *zone;
    ThreadWait *twait;
};


#endif
