/*
    Posadis - A DNS Server
    Functions for reading configuration files
    Copyright (C) 2001  Meilof Veeningen <meilof@users.sourceforge.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef __LEXFN_H
#define __LEXFN_H

#include <stdio.h>

//! Class which encapsulates a very simple lexical analyzer
/*! The posadis_infile class is a class which handles all low-level file
    access and lexical problems for the master file and configuration
    readers.
*/

class posadis_infile {
  public:
	//! Empty constructor
	/*! You should construct a posadis_infile using this empty constructor,
	    followed by a open call to actually open the file. */
	posadis_infile();
	
	//! Destructor
	~posadis_infile();
	
	//! Opens a file and initializes the lexical analyzer
	bool open(char *file);
	
	//! Closes a file
	bool close();

	//! Checks whether we currently have an open file	
	bool is_open();
	
	//! Reads an configuration file part.
	/*! This is one word on a line (or multiple words between "s. If the word
	    doesn't fit, only reslen characters will be returned, but the lexical
	    analyzer will skip to the next entry. */
	int read_entry(char *buff, int bufflen);
	
	//! Checks whether the end-of-line has been reached
	bool is_eol();
	
	//! Checks whether the end-of-file has been readhed
	bool is_eof();

	//!	Jumps to a new line
	bool start_line();
	
	//! Previews a char of the next code (or '\0' if eoc, eol, or eof)
	char preview_char();
	
	//! The current line number
	int linenum;
	
	//! The current file name
	char *name;
	
	//! The file object
	FILE *file;
	
	//! internal: whether we are in parentheses
	bool in_parentheses;
	
	//! The last error code
	enum {
	    ERR_NOPE,
		ERR_EOL,
		ERR_EOF,
		ERR_EOF_IN_QUOTES,
		ERR_EOF_IN_PARENTHESES,
		ERR_EOF_IN_UNESCAPE,
		ERR_UNMATCHED_PARENTHESES,
	} errcode;
	//! Gets a more verbose error message
	char *geterrmsg();
	
	//! Whether we're currently within a line
	bool in_line;
};

void write_long(char *res, int val);
int txt_to_email(char *target, char *src, char *origin = NULL);
int txt_to_dname(char *target, char *src, char *origin = NULL);
int txt_to_text(char *target, char *src);
long txt_to_long(char *buff);
int txt_to_ip(char ip[4], char *buff);
int txt_to_ipv6(char ipv6[16], char *buff);

#endif /* ifndef __LEXFN_H */

