/*
    Posadis - A DNS Server
    Logfile implementation
    Copyright (C) 2001  Meilof Veeningen <meilof@users.sourceforge.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <stdarg.h>
#include <include/unifile.h>
#include <include/unistring.h>
#include <contrib/vsnprintf.h>
#include <time.h>

#include "log.h"
#include "configuration.h"
#include <config.h>

#ifdef _WIN32
#include "w32/w32dialog.h"
#endif

FILE *logfile = NULL;
bool no_stdout_log = false;

void log_init() {
    char filename[MAXFILE];
    strcpy(filename, config.datadir);
    strcat(filename, "log.txt");
    logfile = fopen(filename, "a");
#ifdef HAVE_SYSLOG_H
	openlog("posadis", LOG_PID, LOG_DAEMON);
#endif
}

void log_print(message_log_level log_level, char *logmsg, ...) {
	char buff[4096];
	long tsecs;
	struct tm *tstruct;
	va_list args;
	
	/* compile buffer */	
	tsecs = time(NULL);
	tstruct = localtime(&tsecs);
	sprintf(buff, "%04d/%02d/%02d %02d:%02d:%2d|", tstruct->tm_year + 1900, tstruct->tm_mon + 1, tstruct->tm_mday, tstruct->tm_hour, tstruct->tm_min, tstruct->tm_sec);
	switch (log_level) {
		case LOG_LEVEL_INFO:    strcat(buff, "INFO: "); break;
		case LOG_LEVEL_WARNING: strcat(buff, "WARNING: "); break;
		case LOG_LEVEL_ERROR:   strcat(buff, "ERROR: "); break;
		case LOG_LEVEL_PANIC:   strcat(buff, "PANIC: "); break;
	}
	
	va_start(args, logmsg);
	vsnprintf(&buff[strlen(buff)], sizeof(buff) - strlen(buff) - 1, logmsg, args);
	va_end(args);
	
	/* and print it to various targets */
	if (!no_stdout_log) printf("%s\n", buff);
	if (logfile) fprintf(logfile, "%s\n", buff);
#ifdef _WIN32
	w32dlg_add_log_item(buff);
#endif
#ifdef HAVE_SYSLOG_H
	syslog(log_level, "%s", strchr(buff, '|') + 1);	
#endif
}

void log_close() {
#ifdef HAVE_SYSLOG_H
	closelog();
#endif
    if (logfile) { fclose(logfile); logfile = NULL; }
}

