/*
    Posadis - A DNS Server
    Logfile implementation
    Copyright (C) 2001  Meilof Veeningen <meilof@users.sourceforge.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef __LOG_H
#define __LOG_H

#include <config.h>

//! The Posadis logging mechanism
/*!
  Posadis uses a very simple logging mechanism. If a function wants to print a log message, it uses the log_print function,
  which takes a severity argument, which indicates how bad this error is (info, warning, error, panic).

  Currently, log messages are written to stdout and a file called 'log.txt' in the Posadis 'data' directory.
*/

#ifdef HAVE_SYSLOG_H
#include <syslog.h>
#else
#define LOG_INFO 0
#define LOG_WARNING 1
#define LOG_ERR 2
#define LOG_CRIT 3
#endif

enum message_log_level {
    LOG_LEVEL_INFO = LOG_INFO,       /* nothing serious: only a warning */
    LOG_LEVEL_WARNING = LOG_WARNING, /* something was probably wrong, but we can go on without too much problems */
    LOG_LEVEL_ERROR = LOG_ERR,       /* some error caused a part of the configuration to be discarded, but we continue anyway */
    LOG_LEVEL_PANIC = LOG_CRIT,      /* some unrecoverable error has been found */
};

void log_init();                            // initializes the logfile
void log_print(message_log_level log_level, char *logmsg, ...);
                                            // prints a message to the logfile (new, more sophisticated method
void log_close();                           // closes the logfile

extern bool no_stdout_log;					// set to true by Posadis after forking

#endif /* ifndef __LOG_H */

