/*
    Posadis - A DNS Server
    The RR center - a universal way of handeling RR's
    Copyright (C) 2001  Meilof Veeningen <meilof@users.sourceforge.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "rr.h"
#include "dnsconstants.h"

#include <include/unistring.h>

const int SECTION_PROC_NONE = 1,
          SECTION_PROC_A = 2;

const rr_type rr_types[] = {
    { "A",      1,  "i",        SECTION_PROC_NONE,    true,   false  },
    { "NS",     2,  "d",        SECTION_PROC_A,       true,   true   },
    { "MD",     3,  "d",        SECTION_PROC_A,       true,   true   },
    { "MF",     4,  "d",        SECTION_PROC_A,       true,   true   },
    { "CNAME",  5,  "d",        SECTION_PROC_A,       true,   true   },
    { "SOA",    6,  "dmlllll",  SECTION_PROC_NONE,    true,   true   },
    { "NULL",   10, "f",        SECTION_PROC_NONE,    true,   false  },
    { "WKS",    11, "iw",       SECTION_PROC_NONE,    true,   false  },
    { "PTR",    12, "d",        SECTION_PROC_NONE,    true,   true   },
    { "HINFO",  13, "cc",       SECTION_PROC_NONE,    true,   false  },
    { "MX",     15, "sd",       SECTION_PROC_A,       true,   true   },
    { "TXT",    16, "h",        SECTION_PROC_NONE,    true,   false  },
    { "RP",     17, "md",       SECTION_PROC_NONE,    true,   false  },
    { "AFSDB",  18, "sd",       SECTION_PROC_A,       true,   false  },
    { "PX",     26, "sdd",      SECTION_PROC_NONE,    true,   false  },
    { "AAAA",   28, "6",        SECTION_PROC_NONE,    true,   false  },
    { "SRV",    33, "sssd",     SECTION_PROC_A,       true,   false  },
    { "NAPTR",  35, "sscccd",   SECTION_PROC_NONE,    true,   false  },
    { "A6",     38, "7",        SECTION_PROC_A,       true,   false  },
    { "DNAME",  39, "d",        SECTION_PROC_A,       true,   false  },
};

const int N_RR_TYPES = sizeof(rr_types) / sizeof(rr_type);


rr_type *rrtype_getinfo(u_short type) {
 	int t;
 	for (t = 0; t < N_RR_TYPES; t++)
 		if (rr_types[t].type == type) return (rr_type *)&rr_types[t];
 		
 	return NULL;
}

rr_type *rrtype_getinfo(char *name) {
 	int t;
 	for (t = 0; t < N_RR_TYPES; t++)
 		if (strcmpi(rr_types[t].name, name) == 0) return (rr_type *)&rr_types[t];
 		
 	return NULL;
}

bool support_type(u_short type) {
	int t;
	
    for (t = 0; t < N_RR_TYPES; t++)
    	if (rr_types[t].type == type) return true;
    	
    return false;
}


