/*
    Posadis - A DNS Server
    System socket functions for platform independance
    Copyright (C) 2001  Meilof Veeningen <meilof@users.sourceforge.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef __SYSSOCKET_H
#define __SYSSOCKET_H

#include <include/unisock.h>
#include "dnsconstants.h"

int initsocklib();                          // initialises the socket-library (required for mswinsock)
int closesocklib();                         // closes the socket-library

int tcpcreateserver(_addr *addr);          // creates a tcp server, returning the sockid, or < 0 if failed (port=0 -> random port)
int tcpopen(_addr *addr);					// opens a tcp client connection to <addr>
int tcpaccept(int socket, _addr *addr = NULL); // accepts a new tcp socket connection, giving back the querying address when asked for
int tcpsend(int socket, char *msg, int msglen); // sends a message through tcp, returning the number of bytes sent or <0 if errors
int tcpread(int socket, char *buff, int bufflen); // reads a tcp message into the buffer, returning the number of bytes read or <0 if errors
int tcpclose(int socket);                   // closes a socket connection

int udpcreateserver(_addr *addr);           // creates a udp server, returning the sockid, or < 0 if failed (port=0 -> random port)
int udpread(int socket, char *data, _addr *addr); // reads an udp message into the buffer (sized UDP_MSG_SIZE), returning the number of bytes read or  <0 if errors
int udpsend(int socket, _addr *addr, char *msg, int msglen); // sends a message through udp, returning the number of bytes sent or <0 if errors
int udpclose(int socket);					// closes connection

int watchset_init();                        // initializes the list of sockets to watch (typically a fd_set)
int watchset_add(int sockid);               // adds a sockid to the watchset
int watchset_remove(int sockid);            // removes a sockid from the watchset

int watchset_wait(int maxtime);             // waits until maxime seconds until new data from the watchset arrives, or -1 to wait forever
int watchset_waitread(int maxtime);
int watchset_waitforever();                 // waits until some data can be read or some error has occured
int watchset_check();                       // calls watchset_wait(0) to only check which connections have data
                                            // all three functions return the number of ready descriptors
                                            
bool watchset_isdata(int sockid);           // determines whether data has arrived on the sockid
bool watchset_iserror(int sockid);          // determines whether an error has occured on the sockid
bool watchset_canwrite(int sockid);			// determines whether a write call will be nonblocking

int getprotocolbyname(char *name);          // returns the protocol number of the specified protocol name (for example 'tcp' or 'udp). used for wks rdata
int getserviceportbyname(char *name);       // returns the port name of the service with the specified name (for wks rdata)

bool tcp_is_open(int sockid);               // check whether someone's still connected on the other end of the line
int gettcpaddr(int sockid, _addr *addr);    // returns the address of the person on the other end of the line
int getaddress(_addr *res, char *ip = "0.0.0.0", int port = DNS_PORT); // generic address translation function
int getaddress_ip4(_addr *res, char *ipv4_data, int port = DNS_PORT); // ipv4 raw data address translation
int getaddress_ip6(_addr *res, char *ipv6_data, int port = DNS_PORT); // ipv6 raw data address translation
int answeraddress(_addr *res, _addr *src, int port = -1); // clones an address, setting the port if greater than zero

bool is_ipv4(_addr *addr);
#ifdef HAVE_IPV6
bool is_ipv6(_addr *addr);
#endif

char *addr_to_string(_addr *addr); // gets text representation of structure

bool compare_hostport(_addr *addr1, _addr *addr2); // returns true if both structures have the same adresses
bool compare_host(_addr *addr1, _addr *addr2); // returns true if both structures have the same adress

bool address_lookup(_addr *res, char *name, int port = DNS_PORT); // looks up using local resolver

#endif /* ifndef __SYSSOCKET_H */

