/*
    Posadis - A DNS Server
    Tunneling
    Copyright (C) 2001  Meilof Veeningen <meilof@users.sourceforge.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef __TUNNEL_H
#define __TUNNEL_H

#include "classes.h"
#include "zones.h"
#include "threads.h"
#include "vector.h"
#include "hint.h"

/* tunnel zone group */

class TunnelGroup : public HintGroup { public: TunnelGroup(); tunnel_configuration_data tunnel_config; };

/*!
  The TunnelZone is a special kind of zone: if a query is recieved for such
  a zone, it will be redirected to another nameserver, and the answer of that
  nameserver will be copied.

  This class is a subclass of Zone. It doesn't currently contain any special
  behavior in the zone itself.
*/

class TunnelZone : public HintZone {
  public:
    TunnelZone(_domain domain);                         // public constructor
    virtual ~TunnelZone();                              // guess what? it's a destructor!

    virtual void start_updating();						// update data
    virtual void offer_glue(CacheItem *item);           // gets glue

    tunnel_configuration_data tunnel_config;            // some settings
    virtual _thread_answer do_lookup_thread(AnsweringThread*& thr, stdquery_info *info, _domain QNAME);
	
	Vector<_addr *> tunnelservers;						// cached list of ip numbers of possible servers
};

enum _tunnelstate {
	T_NOTHING,								/* nothing special */
	T_SENDING,								/* sending a request */
	T_WAITING,								/* waiting for an answer */
};

class TunnelThread : public AnsweringThread {
  public:
	TunnelThread(stdquery_info *info, _domain QNAME, TunnelZone *z); // contructor
	virtual ~TunnelThread();				// destructor
	
	virtual int execute();					// overrided
	
  protected:
	TunnelZone *zone;						// the zone in which the data reside
    stdquery_info *info;
	int attemptix;							// the attempt index
	
	ThreadWait *twait;						// structure for data request
	
	_tunnelstate state;                     // internal status
	Thread *child;							// a child thread
	
	long endtime;                           // time before our attempt ends	
};


#endif /* __TUNNEL_H */
