/*
    Posadis - A DNS Server
    Handles the information on zones we're authoritive for
    Copyright (C) 2001  Meilof Veeningen <meilof@users.sourceforge.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <include/unifile.h>
#include <include/unistring.h>
#include <config.h>

#include "bitlist.h"
#include "zones.h"
#include "cachelookup.h"
#include "log.h"
#include "posadis.h"
#include "tunnel.h"
#include "authoritive.h"
#include "hint.h"
#include "tunnel.h"

int get_configuration_data(configuration_data **settings, Zone *zone) {
    switch (zone->type) {
        case ZONE_PRIMARY:
            settings[0] = &zone->zone_config;
            settings[1] = &((PrimaryZone*)zone)->auth_config;
            settings[2] = &((PrimaryZone*)zone)->primary_config;
            settings[3] = NULL;
            break;
        case ZONE_SECONDARY:
            settings[0] = &zone->zone_config;
            settings[1] = &((SecondaryZone*)zone)->auth_config;
            settings[2] = &((SecondaryZone*)zone)->secondary_config;
            settings[3] = NULL;
            break;
        case ZONE_HINT:
            settings[0] = &zone->zone_config;
            settings[1] = &((HintZone*)zone)->hint_config;
            settings[2] = NULL;
            break;
        case ZONE_TUNNEL:
            settings[0] = &zone->zone_config;
            settings[1] = &((HintZone*)zone)->hint_config;
            settings[2] = &((TunnelZone*)zone)->tunnel_config;
            settings[3] = NULL;
            break;
        default:
            settings[0] = NULL;
    }
    return 0;
}

int get_configuration_data(configuration_data **settings, ZoneGroup *group) {
    switch (group->type) {
        case ZONE_PRIMARY:
            settings[0]=&group->config;
            settings[1]=&((PrimaryGroup*)group)->auth_config;
            settings[2]=&((PrimaryGroup*)group)->primary_config;
            settings[3]=NULL;
            break;
        case ZONE_SECONDARY:
            settings[0]=&group->config;
            settings[1]=&((SecondaryGroup*)group)->auth_config;
            settings[2]=&((SecondaryGroup*)group)->secondary_config;
            settings[3]=NULL;
            break;
        case ZONE_HINT:
            settings[0]=&group->config;
            settings[1]=&((HintGroup*)group)->hint_config;
            settings[2] = NULL;
            break;
        case ZONE_TUNNEL:
            settings[0]=&group->config;
            settings[1]=&((HintGroup*)group)->hint_config;
            settings[2]=&((TunnelGroup*)group)->tunnel_config;
            settings[3] = NULL;
            break;
        default:
            settings[0] = NULL;
    }
    return 0;
}

int inherit_group_settings(configuration_data **res, ZoneGroup *group) {
    configuration_data* settings[4];
    configuration_data **cursrc, **curres;

    get_configuration_data(settings, group);

    cursrc = &(settings[0]); curres = &(res[0]);
    while (*cursrc) {
        (*curres)->inherit(*cursrc);
        curres++;
        cursrc++;
    }

    return 0;
}

/* initialize root zone settings */
void init_default_rootzone(Zone *newzone) {
        HintZone *root = (HintZone *)newzone;
        char info[25];
        
        root->hint_config.override_p(&root->hint_config.nameserver);
        
        memcpy(info, "\1A\xcROOT-SERVERS\3NET\0\1", 21); txt_to_ip(&info[21], "192.41.0.4");
        multi_add(&root->hint_config.nameserver, (intptr_t) memdup(info, 25));  
        memcpy(info, "\1B\xcROOT-SERVERS\3NET\0\1", 21); txt_to_ip(&info[21], "128.9.0.107");
        multi_add(&root->hint_config.nameserver, (intptr_t) memdup(info, 25));  
        memcpy(info, "\1C\xcROOT-SERVERS\3NET\0\1", 21); txt_to_ip(&info[21], "192.33.4.12");
        multi_add(&root->hint_config.nameserver, (intptr_t) memdup(info, 25));  
        memcpy(info, "\1D\xcROOT-SERVERS\3NET\0\1", 21); txt_to_ip(&info[21], "128.8.10.90");
        multi_add(&root->hint_config.nameserver, (intptr_t) memdup(info, 25));  
        memcpy(info, "\1E\xcROOT-SERVERS\3NET\0\1", 21); txt_to_ip(&info[21], "192.203.230.10");
        multi_add(&root->hint_config.nameserver, (intptr_t) memdup(info, 25));  
        memcpy(info, "\1F\xcROOT-SERVERS\3NET\0\1", 21); txt_to_ip(&info[21], "192.5.5.241");
        multi_add(&root->hint_config.nameserver, (intptr_t) memdup(info, 25));  
        memcpy(info, "\1G\xcROOT-SERVERS\3NET\0\1", 21); txt_to_ip(&info[21], "192.112.36.4");
        multi_add(&root->hint_config.nameserver, (intptr_t) memdup(info, 25));  
        memcpy(info, "\1H\xcROOT-SERVERS\3NET\0\1", 21); txt_to_ip(&info[21], "128.63.2.53");
        multi_add(&root->hint_config.nameserver, (intptr_t) memdup(info, 25));  
        memcpy(info, "\1I\xcROOT-SERVERS\3NET\0\1", 21); txt_to_ip(&info[21], "192.36.148.17");
        multi_add(&root->hint_config.nameserver, (intptr_t) memdup(info, 25));  
        memcpy(info, "\1J\xcROOT-SERVERS\3NET\0\1", 21); txt_to_ip(&info[21], "198.41.0.10");
        multi_add(&root->hint_config.nameserver, (intptr_t) memdup(info, 25));  
        memcpy(info, "\1K\xcROOT-SERVERS\3NET\0\1", 21); txt_to_ip(&info[21], "193.0.14.129");
        multi_add(&root->hint_config.nameserver, (intptr_t) memdup(info, 25));  
        memcpy(info, "\1L\xcROOT-SERVERS\3NET\0\1", 21); txt_to_ip(&info[21], "198.32.64.12");
        multi_add(&root->hint_config.nameserver, (intptr_t) memdup(info, 25));  
        memcpy(info, "\1M\xcROOT-SERVERS\3NET\0\1", 21); txt_to_ip(&info[21], "202.12.27.33");
        multi_add(&root->hint_config.nameserver, (intptr_t) memdup(info, 25));  
}

#define line_read_error(errcode)  {											\
	cptr = file.geterrmsg(); 												\
	log_print(LOG_LEVEL_ERROR, "%s:%d: %s", file.name, file.linenum, cptr);	\
	free(cptr);																\
	return -1;																\
	}

int read_zone_settings(configuration_data **config_settings, posadis_infile& file) {
    char item[256];
    configuration_data **curc;
    setting *ptr;
    int ret;
    char *cptr;

    while (!file.is_eol()) {
        ret = file.read_entry(item, sizeof(item));
  	    if (ret < 0) line_read_error(ret);

  	    curc = &config_settings[0];
  	    while (*curc) {  	
      	    ptr = (*curc)->getcommandinfo(item);
  	        if (ptr) {
  	            ret = handle_config_item(file, ptr, *curc);
  	            if (ret < 0) return ret;
				break;
  	        }
  	        curc++;
  	    }
  	    if (!*curc) {
  	        log_print(LOG_LEVEL_ERROR, "%s:%d: Unrecognized option %s!", file.name, file.linenum, item);
  	        return -1;
  	    }
  	}
  	return 0;
}

int read_zone_line(posadis_infile& file) {
    /* usage: zone <type> <domain> <props> */
    char item[256], name[256], domain[256];
    Zone *newzone, *otherzone;
    configuration_data* settings[5];
    int ret, x;
    char *cptr;

    /* read <type> */
    ret = file.read_entry(item, sizeof(item));
  	if (ret < 0) line_read_error(ret);
  	
  	/* read <domain> */
    ret = file.read_entry(name, sizeof(name));
  	if (ret < 0) line_read_error(ret);
  	
  	ret = txt_to_dname(domain, name);
  	if (ret < 0) {
  	    log_print(LOG_LEVEL_ERROR, "%s:%d: Incorrect domain name for zone!", file.name, file.linenum);
  	    return -1;
  	}
  	
  	/* initialize the zone */
    if (strcmpi(item, "primary") == 0) {
        newzone = (Zone *)new PrimaryZone(domain);
    } else if (strcmpi(item, "secondary") == 0) {
        newzone = (Zone *)new SecondaryZone(domain);
    } else if (strcmpi(item, "hint") == 0) {
        newzone = (Zone *)new HintZone(domain);
    } else if (strcmpi(item, "tunnel") == 0) {
        newzone = (Zone *)new TunnelZone(domain);
    } else {
        log_print(LOG_LEVEL_ERROR, "%s:%d: Unrecognized zone %s!", file.name, file.linenum, item);
        return -1;
    }

    get_configuration_data(settings, newzone);

    /* now people can specify the zone groups this zone belongs to, for example:
         group of hint the_group update_ttl 3h
         hint . <the_group ... */

    while (!file.is_eol()) {
        if (file.preview_char() == '<') {
            ret = file.read_entry(item, sizeof(item));
            if (ret < 0) line_read_error(ret);

            for (x = 0; x < ZoneGroups.count; x++) {
                if (strcmp(&item[1], ZoneGroups[x]->name) == 0) {
                    /* found the zone group this zone wants to belong to */

                    /* check whether the zone group type is compatible */
                    if (ZoneGroups[x]->type == newzone->type) {
                        inherit_group_settings(settings, ZoneGroups[x]);
                    } else {
                        domtostring(item, domain);
                        log_print(LOG_LEVEL_ERROR,
                            "%s:%d: The zone %s and the zone group %s are of conflicting types!",
                            file.name, file.linenum, name, ZoneGroups[x]->name);
                        delete newzone; return -1;
                    }
                    break;
                }
            }
            if (x == ZoneGroups.count) {
                log_print(LOG_LEVEL_ERROR, "%s:%d: The zone group %s inherits from could not be found!", file.name, file.linenum, name);
                delete newzone; return -8;
            }
        } else break;
    }

    ret = read_zone_settings(settings, file);
    if (ret < 0) { delete newzone; return ret; }

    /* now try and create the associated subdomain */
    if (domain[0] == '\0') {
        /* the root zone is being defined */
        if (rootzone != NULL) {
        	log_print(LOG_LEVEL_ERROR, "The root zone should be the first zone to be defined!");
        	delete newzone;
        	return -1;
        }
        rootzone = newzone;
    } else {
    	if (rootzone == NULL) {
    		/* no root zone yet */
	     	log_print(LOG_LEVEL_WARNING, "No root zone defined -- defaults will be used.");
    	 	rootzone = new HintZone("");
			init_default_rootzone(rootzone);
    	}
    	/* add the zone to the appropriate superzone */
    	ret = LookupZone(otherzone, newzone->domain);
    	otherzone->subzones.add(newzone);
    }
    return 0;
}

int read_group_line(posadis_infile& file) {
    /* usage: group of <type> <groupname> */
    char item[256], name[256];
    ZoneGroup *newgroup;
    configuration_data* settings[5];
    int ret;
    char *cptr;

    /* read 'of' */
    ret = file.read_entry(item, sizeof(item));
  	if (ret < 0) line_read_error(ret);

  	if (strcmpi(item, "of") != 0) {
  	    log_print(LOG_LEVEL_ERROR, "%s:%d: Expected 'of' after 'group'", file.name, file.linenum);
  	    return -1;
  	}
  	
  	/* read <type> */
    ret = file.read_entry(item, sizeof(item));
  	if (ret < 0) line_read_error(ret);
  	
  	/* and <name> */
  	ret = file.read_entry(name, sizeof(name));
  	if (ret < 0) line_read_error(ret);

    if (strcmpi(item, "auth") == 0) {
        newgroup = new AuthGroup();
    } else if (strcmpi(item, "primary") == 0) {
        newgroup = new PrimaryGroup();
    } else if (strcmpi(item, "secondary") == 0) {
        newgroup = new SecondaryGroup();
    } else if (strcmpi(item, "hint") == 0) {
        newgroup = new HintGroup();
    } else if (strcmpi(item, "tunnel") == 0) {
        newgroup = new TunnelGroup();
    } else {
        log_print(LOG_LEVEL_ERROR, "%s:%d: Unknown grouptype %s!", file.name, file.linenum, item);
        return -1;
    }

    get_configuration_data(settings, newgroup);
    newgroup->name = strdup(name);

    ret = read_zone_settings(settings, file);
    if (ret < 0) { delete newgroup; return ret; }

    ZoneGroups.add(newgroup);
    return 0;
}

int recursive_start_update(Zone *zone) {
	int x;
	zone->start_updating();
	for (x = 0; x < zone->subzones.count; x++)
		recursive_start_update(zone->subzones[x]);
	return 0;
}

int zones_init() {
    if (rootzone == NULL) {
        /* the root has not been found, we set the default: a hint zone
           withouy any nameservers */
     	/* no root zone defined! */
     	log_print(LOG_LEVEL_WARNING, "No root zone defined -- defaults will be used.");
     	rootzone = new HintZone("");
		init_default_rootzone(rootzone);
    }
    /* now that we know of all the zones, we give them the chance to update themselves */

    recursive_start_update(rootzone);

    return 0;

}

/* ----- the zone classes --------------------------------------------------------------------------------- */

Vector<stdquery_info *> current_std_queries;

stdquery_info::stdquery_info(DnsMessage *_query, DnsMessage *_answer, _addr *_querier, MainThread* _tr, int _QTYPE) {
    query = _query;
    answer = _answer;
    tr = _tr;
    QTYPE = _QTYPE;
    is_first = true;
    querier = _querier;
    current_std_queries.add(this);
}

stdquery_info::~stdquery_info() {
     current_std_queries.removeitem(this, false);
}

xfr_info::xfr_info(DnsMessage *_q, DnsMessage *_a, _addr *_qu, MainThread *_tr) {
    query = _q;
    answer = _a;
    querier = _qu;
    tr = _tr;
}

xfr_info::~xfr_info() { }

Vector<ZoneGroup*> ZoneGroups;
Zone *rootzone = NULL;

/* the basic superclass Zone */
Zone::Zone(char *_domain) {
    domain = domdup(_domain);
    citem_root = new CacheItem(_domain);
    is_being_updated = false;
    zone_is_loaded = true;
}

Zone::~Zone() {
    free(domain);
    delete citem_root;
}

void Zone::start_updating() {
    next_update = 0; /* don't ever update! */
}

void Zone::offer_glue(CacheItem *item) {
    /* we don't care */
}

_thread_answer Zone::do_lookup_thread(AnsweringThread*& thr, stdquery_info *info, _domain QNAME) {
    return do_nonsupported_thread(thr, info->query, info->answer, info->tr);
}


_thread_answer Zone::do_axfr_thread(AnsweringThread*& thr, xfr_info *info) {
	return do_nonsupported_thread(thr, info->query, info->answer, info->tr);
}

_thread_answer Zone::do_ixfr_thread(AnsweringThread*& thr, xfr_info *info) {
	return do_nonsupported_thread(thr, info->query, info->answer, info->tr);
}

_thread_answer Zone::do_notify_thread(AnsweringThread*& thr, DnsMessage *q, _addr *requestor, DnsMessage *a, MainThread *tr) {
	return do_nonsupported_thread(thr, q, a, tr);
}

_thread_answer Zone::do_nonsupported_thread(AnsweringThread*& thr, DnsMessage *q, DnsMessage *a, MainThread *tr) {
    if (a->question.count == 0) a->answer_init(q, false);
    a->RCODE = RCODE_NOTIMPLEMENTED;
    return TA_SUCCESS;
}


_thread_answer global_do_lookup_thread(AnsweringThread*& thr, stdquery_info *info, _domain QNAME) {
    Zone *zn;
    _thread_answer ret;

    if (info->answer->answer.count == 0 &&
        info->answer->authority.count == 0 &&
        info->answer->additional.count == 0)
        info->is_first = true;
    else
        info->is_first = false;

    LookupZone(zn, QNAME);
    if (!zn->zone_is_loaded) {
        info->answer->RCODE = RCODE_SERVERFAILURE;
        free(QNAME);
        return TA_SUCCESS;
    }

    ret = zn->do_lookup_thread(thr, info, QNAME);
    free(QNAME);
    return ret;
}

_thread_answer global_do_axfr_thread(AnsweringThread*& thr, xfr_info *info) {
    Zone *zn;
    _thread_answer ret;
	
    info->answer->answer_init(info->query, false);

    if (info->query->question.count != 1 ||
        info->query->answer.count ||
        info->query->authority.count ||
        info->query->additional.count) {
        info->answer->RCODE = RCODE_QUERYERROR;
        return TA_SUCCESS;
    }

    char *QNAME = info->query->question[0]->QNAME;
    LookupZone(zn, QNAME);
    ret = zn->do_axfr_thread(thr, info);
    delete info;
    return ret;
}

_thread_answer global_do_ixfr_thread(AnsweringThread*& thr, xfr_info *info) {
    Zone *zn;
    _thread_answer ret;

    info->answer->answer_init(info->query, false);

    if (info->query->question.count != 1 ||
        info->query->answer.count ||
        info->query->authority.count != 1 ||
        info->query->additional.count) {
        info->answer->RCODE = RCODE_QUERYERROR;
        return TA_SUCCESS;
    }

    char *QNAME = info->query->question[0]->QNAME;
    LookupZone(zn, QNAME);
    ret = zn->do_ixfr_thread(thr, info);
    delete info;
    return ret;
}

_thread_answer global_do_notify_thread(AnsweringThread*& thr, DnsMessage *q, _addr *requestor, DnsMessage *a, MainThread *tr) {
    Zone *zn;
    char *QNAME;

    if (q->question.count != 1 ||
        q->question[0]->QTYPE != DNS_TYPE_SOA ||
        q->question[0]->QCLASS != CLASS_IN) {
        a->RCODE = RCODE_QUERYERROR;
        return TA_SUCCESS;
    } else {
        a->answer_init(q);
        QNAME = q->question[0]->QNAME;
        LookupZone(zn, QNAME);
        return zn->do_notify_thread(thr, q, requestor, a, tr);
    }
}

/* NotauthZone: superclass for zones we're not authoritive for */
NotauthZone::NotauthZone(_domain _dom) : Zone(_dom) { }
NotauthZone::~NotauthZone() { }

/* Zone Threads ---------------------------------------------------------------------------------- */

/* the zones thread updates zones when required */
ZonesThread *zones_thread; /* gets cleaned up by Vector and initialized by main */

ZonesThread::ZonesThread() : MainThread() {
    calc_next();
    wakeup_time = curtime;
    trstate = THREAD_SLEEP;
}

int ZonesThread::execute() {
    if (nexttime && curtime >= nexttime) zone->start_updating();
    calc_next();
    wakeup_time = curtime + 60;
    if (nexttime && nexttime < wakeup_time) wakeup_time = nexttime;
    trstate = THREAD_SLEEP;
    return TRET_OKBUSY;
}

int get_next_zone(Zone* zone, Zone*& retzone) {
	int t, nexttime = 0, retnext;
	Zone* tmp;
	if (zone->next_update && (!nexttime || zone->next_update < nexttime)) {
		nexttime = zone->next_update;
		retzone = zone;
	}
	for (t = 0; t < zone->subzones.count; t++) {
		retnext = get_next_zone(zone->subzones[t], tmp);
		if (retnext && (!nexttime || retnext < nexttime)) {
			nexttime = retnext;
			retzone = tmp;
		}
	}
	return nexttime;
}

void ZonesThread::calc_next() {
	nexttime = get_next_zone(rootzone, zone);
}

