/*
 * Posask - Posadis Address from Server Kit
 *
 * Copyright (C) Meilof Veeningen, 2002
 */
 
#include <query.h>
#include <syssocket.h>
#include <lexfn.h>
#include <dnsconstants.h>
#include <version.h>
#include <rr.h>
#include <domainfn.h>

#include <include/unistring.h>
#include <include/unireadline.h>
#include <include/unitime.h>

#ifndef _WIN32
#include <unistd.h>
#endif
#include <stdlib.h>
#include <signal.h>

_addr server;
char dname[256] = "";
int dname_given = 0;
int qtype = 0;
int serial = 0;
int rd = 1;
int use_tcp = -1; /* default */
int timeout = 5;

query_info conn;

int cmd_server(char *line);
int cmd_dname(char *line);
int cmd_qtype(char *line);
int cmd_serial(char *line);
int cmd_rd(char *line);
int cmd_tcp(char *line);
int cmd_timeout(char *line);
int cmd_query(char *line);
int cmd_set(char *line);
int cmd_help(char *line);
int cmd_exit(char *line);

typedef int(*commandfn)(char *);
typedef char*(*completefn)(const char *, int);

char *no_completion(const char *, int val);
char *command_completion(const char *word, int val);
char *bool_completion(const char *word, int val);
char *setting_completion(const char *word, int val);

char q_number[16];

char *str_qtype(int qtype) {
	if (qtype == QTYPE_AXFR) return "AXFR";
    if (qtype == QTYPE_IXFR) return "IXFR";
	if (qtype == QTYPE_ALL) return "any";
	rr_type *rrtype = rrtype_getinfo(qtype);
	if (!rrtype) { sprintf(q_number, "%d", qtype & 65535); return q_number; }
	return rrtype->name;
}

char o_number[16];

char *str_opcode(int opcode) {
	if (opcode == OPCODE_QUERY) return "QUERY";
    sprintf(o_number, "%d", opcode & 65535);
    return o_number;
}

char r_number[16];

char *str_rcode(int rcode) {
	if (rcode == RCODE_NOERROR) return "NOERROR";
	if (rcode == RCODE_QUERYERROR) return "QUERYERR";
	if (rcode == RCODE_SERVERFAILURE) return "SRVFAIL";
	if (rcode == RCODE_NXDOMAIN) return "NXDOMAIN";
	if (rcode == RCODE_NOTIMPLEMENTED) return "NOTIMP";
	if (rcode == RCODE_REFUSED) return "REFUSED";
    sprintf(r_number, "%d", rcode & 65535);
    return r_number;
}

void print_qitem(question_item *item) {
	char tmp[256];
	domtostring(tmp, item->QNAME);
	printf(";%-20s%s\n", tmp, str_qtype(item->QTYPE));
}

void print_aitem(complete_rr *rr) {
	char *prop, *pos;
	AgBuffer buff;
	char tmp[512];
	rr_type *info = rrtype_getinfo(rr->TYPE);
	int x, c;

	domtostring(tmp, rr->NAME);
	printf("%-20s %-6d %-6s", tmp, rr->TTL, str_qtype(rr->TYPE));

	if (!info) {
		printf("<unknown>\n");
	} else {
		prop = info->properties;
		pos = rr->RDATA;
		while (*prop && pos < (rr->RDATA + rr->RDLENGTH)) {
			switch(*prop) {
				case 'd':
				case 'm':
					domtostring(tmp, pos);
					pos += domlen(pos);
					break;
				case 'i':
					sprintf(tmp, "%d.%d.%d.%d", pos[0], pos[1], pos[2], pos[3]);
					pos += 4;
					break;
				case 's':
					sprintf(tmp, "%d", pos[0] * 256 + pos[1]);
					pos += 2;
					break;
				case 'l':
					sprintf(tmp, "%d", pos[0] * 16777216 + pos[1] * 65536 + pos[2] * 256 + pos[3]);
					pos += 4;
					break;
				case 'c':
					tmp[0] = '"';
					memcpy(&tmp[1], pos + 1, *pos);
					tmp[*pos + 1] = '"';
					tmp[*pos + 2] = '\0';
					pos += *pos + 1;
					break;
				case 'h':
					while (pos < (rr->RDATA + rr->RDLENGTH)) {
						buff.catchar('"');
						buff.cat(pos + 1, *pos);
						buff.catchar('"');
                        buff.catchar(' ');
						pos += *pos + 1;
					}
					buff.catchar(' ');
					tmp[0] = '\0';
					break;
				case 'w':
					c = 0;
					sprintf(tmp, "%d:", *(pos++));
					while (pos < (rr->RDATA + rr->RDLENGTH)) {
						for (x = 0; x < 8; x++)
							if (bitisset(pos, x)) sprintf(&tmp[strlen(tmp)], " %d", c + x);
						c += 8;
						pos++;
					}
					break;
				case '6':
					sprintf(tmp, "%x%x:%x%x:%x%x:%x%x:%x%x:%x%x:%x%x:%x%x",
						pos[0], pos[1], pos[2], pos[3], pos[4], pos[5], pos[6], pos[7],
						pos[8], pos[9], pos[10], pos[11], pos[12], pos[13], pos[14], pos[15]);
					pos += 16;
					break;
        case '7':
          /* ipv6 thingy */
          char tmp6[16];
          memset(tmp6, 0, sizeof(tmp6));
          bitstringcpy(tmp6, pos[0], pos + 1, 0, 128 - pos[0]);
          sprintf(tmp, "%d %x%x:%x%x:%x%x:%x%x:%x%x:%x%x:%x%x:%x%x ", (int)pos[0],
						tmp6[0], tmp6[1], tmp6[2], tmp6[3], tmp6[4], tmp6[5], tmp6[6], tmp6[7],
						tmp6[8], tmp6[9], tmp6[10], tmp6[11], tmp6[12], tmp6[13], tmp6[14], tmp6[15]);
          tmp6[0] = pos[0];
          pos += ((135 - pos[0]) / 8) + 1;
          if (tmp6[0] != 0) {
            /* dname follows */
            domtostring(&tmp[strlen(tmp)], pos);
            pos += domlen(pos);
          }
          break;
				default:
					strcpy(tmp, "?");
			}
			buff.catstring(tmp);
			buff.catstring(" ");
			prop++;
		}
		buff.catchar(0);
		printf("%s\n", buff.buff);
	}
}

typedef struct {
	char *command;
	char *help;
	commandfn fn;
	completefn complete;
} _command;

_command commands[] = {
	{ "server", "Sets the server at run-time", cmd_server, no_completion },
	{ "dname", "Sets domain name to query", cmd_dname, no_completion },
	{ "qtype", "Sets the default QTYPE", cmd_qtype, no_completion },
    { "serial", "Sets the previous serial for IXFR requests", cmd_serial, no_completion },
	{ "rd", "Sets whether we demand recursion", cmd_rd, bool_completion },
        { "use_tcp", "Sets whether we use TCP/IP (default only xfr)", cmd_tcp, bool_completion },
	{ "timeout", "Sets the time in seconds to wait for an answer", cmd_timeout, no_completion },
	{ "query", "Sends a query to the nameserver", cmd_query, no_completion },
	{ "q", "Synonym for 'query'", cmd_query, no_completion },
	{ "set", "Show current settings", cmd_set, setting_completion },
	{ "help", "Print this message", cmd_help, command_completion },
	{ "exit", "Quits the program", cmd_exit, no_completion },
};
int n_commands = sizeof(commands) / sizeof(_command);

char* settings[] = { "server", "dname", "qtype", "rd", "use_tcp", "timeout" };
int n_settings = sizeof(settings) / sizeof(char *);

int issue_command(char *name, char *arg) {
	int t;
	for (t = 0; t < n_commands; t++) {
		if (stricmp(name, commands[t].command) == 0) {
			return commands[t].fn(arg);
		}
	}
	
	printf("Command unknown: %s\n", name);
	return -1;
}

int cmd_server(char *line) {
	int port = DNS_PORT;
	char *ptr = strchr(line, '#');

	if (ptr) {
		/* both addr & port */
        *ptr = '\0';
		port = txt_to_long(ptr + 1);
		if (port < 0) {
			*ptr = ':';
			printf("Server %s: no correct port!\n", line);
			return -1;
		}
	}

    if (strchr(line, ':') == NULL &&
        strchr(line, '.') == NULL) {
        /* neither IPv4 nor IPv6 */
        port = txt_to_long(line);
		if (port < 0) {
			printf("Server %s: no correct port!\n", line);
			return -1;
		} else if (port == 0) port = DNS_PORT;
        line = "127.0.0.1";
    }
	
    if (!address_lookup(&server, line, port)) {
		printf("Server %s: no correct identifier!\n", line);
		return -1;
	}
	
	if (ptr) *ptr = ':';
	return 1;
}

int cmd_dname(char *line) {
	dname_given = 1;
	if (txt_to_dname(dname, line) < 0) {
		printf("Incorrect dname %s!\n", line);
		return -1;
	}
	return 1;
}

int cmd_qtype(char *line) {
	rr_type *type;
    int ret;
	
	type = rrtype_getinfo(line);
	if (type == NULL) {
        if (strncmpi(line, "ixfr", 4) == 0) {
            if (line[4] == '/') {
                ret = cmd_serial(&line[5]);
                if (ret > 0) qtype = QTYPE_IXFR;
                return ret;
            } else {
                qtype = QTYPE_IXFR;
                return 1;
            }
        }
		if (strcmpi(line, "axfr") == 0) {
			qtype = QTYPE_AXFR;
			return 1;
		}
		if (strcmpi(line, "any") == 0) {
			qtype = QTYPE_ALL;
			return 1;
		}
		printf("Unknown QTYPE %s!\n", line);
		return -1;
	} else {
		qtype = type->type;
		return 1;
	}
}

int cmd_serial(char *line) {
    int val = txt_to_long(line);

    if (val < 0) {
        printf("The value %s is not a valid serial!", line);
        return -1;
    } else {
        serial = val;
        return 1;
    }
}

int cmd_rd(char *line) {
	if (strcmpi(line, "true") == 0) rd = 1;
	else if (strcmpi(line, "false") == 0) rd = 0;
	else {
		printf("Unrecognized boolean %s in rd!\n", line);
		return -1;
	}
	return 1;
}

int cmd_tcp(char *line) {
	if (strcmpi(line, "true") == 0) use_tcp = 1;
	else if (strcmpi(line, "false") == 0) use_tcp = 0;
	else {
		printf("Unrecognized boolean %s in use_tcp!\n", line);
		return -1;
	}
	return 1;
}

int cmd_timeout(char *line) {
	int x;
	
	x = txt_to_long(line);
	if (x < 0) {
		printf("%s isn't a correct numeric timeout!\n", line);
		return -1;
	}
	
	timeout = x;
	return 1;
}

inline int timediff(struct our_timeval &begintime, struct our_timeval &endtime) {
	return (endtime.tv_sec - begintime.tv_sec) * 1000 +
	       (endtime.tv_usec - begintime.tv_usec) / 1000;
}

int cmd_query(char *line) {
	DnsMessage *q, *a = NULL;
	int x, prevtype = qtype;
    char *tmp;
	_transport trans;
	char *ptr;
	our_timeval begintime, endtime;

	if (line[0]) {
		ptr = &line[strlen(line) - 1];
		while (ptr > line && *ptr != ' ') ptr--;
		if (*ptr == ' ') {
			*ptr = '\0';
			if (cmd_qtype(ptr + 1) < 0) return -1;
		}
		if (cmd_dname(line) < 0) {qtype = prevtype; return -1; }
	}

	gettimeofday(&begintime, NULL);
	q = make_query(dname, qtype);
	qtype = prevtype;
	q->ID = 3000;
    if (qtype == QTYPE_IXFR) {
        /* add a SOA record */
        tmp = (char *)malloc(22);
        memset(tmp, 0, 22);
        write_long(&tmp[2], serial);
        q->authority.add(new complete_rr(domdup(dname), DNS_TYPE_SOA, CLASS_IN, 0, 22, tmp));
    }
    if (rd) q->RD = true;
	if (qtype == QTYPE_AXFR) q->RD = false;
	if (use_tcp == 1 || (use_tcp == -1 && (qtype == QTYPE_AXFR || qtype == QTYPE_IXFR))) {
		trans = PROTO_TCP;
	}
	else
		trans = PROTO_UDP;
	conn.timeout = timeout;
	if (conn.connect(&server, trans) < 0) {
		printf("Could not connect: %s!\n", conn.errmsg);
	} else {
		if (conn.nsquery(q, &a) == 0) {
			printf(
				"; Answer ID: %d  QR: %d  OPCODE: %s  AA: %d  TC: %d  RD: %d\n"
				";        RA: %d  RCODE: %s  qc %d  an %d  au %d  ad %d\n",
				a->ID, a->QR, str_opcode(a->OPCODE), a->AA, a->TC,
				a->RD, a->RA, str_rcode(a->RCODE), a->question.count,
				a->answer.count, a->authority.count, a->additional.count);
			printf("; Questions:\n");
			if (a->question.count) for (x = 0; x < a->question.count; x++) print_qitem(a->question[x]);
			else printf(";(none)\n");
			printf("; Answers:\n");
			if (a->answer.count) for (x = 0; x < a->answer.count; x++) print_aitem(a->answer[x]);
			else printf(";(none)\n");
			printf("; Authority:\n");
			if (a->authority.count) for (x = 0; x < a->authority.count; x++) print_aitem(a->authority[x]);
			else printf(";(none)\n");
			printf("; Additional:\n");
			if (a->additional.count) for (x = 0; x < a->additional.count; x++) print_aitem(a->additional[x]);
			else printf(";(none)\n");
			gettimeofday(&endtime, NULL);
			printf("; Query took:      %d msec\n", timediff(begintime, endtime));
      tmp = addr_to_string(&server);
      printf("; Server queried:  %s", tmp);
      if (trans == PROTO_TCP) printf("[tcp]\n"); else printf("[udp]\n");
      free(tmp);
		} else printf("Query failed: %s!\n", conn.errmsg);
		conn.disconnect();
	}
	delete q;
	if (a) delete a;

	return 1;
}

int cmd_set(char *line) {
	char *tmp = NULL;
	if (!line[0] || strcmpi(line, "server") == 0) {
        tmp = addr_to_string(&server);
        printf("Server:      %s\n", tmp);
        free(tmp);
	}
	if (!line[0] || strcmpi(line, "dname") == 0) {
		domtostring(tmp, dname);
		printf("Domain name: %s\n", tmp);
	}
	if (!line[0] || strcmpi(line, "qtype") == 0)
		printf("QTYPE:       %s\n", str_qtype(qtype));
	if (!line[0] || strcmpi(line, "serial") == 0)
		printf("IXFR serial: %d\n", serial);
	if (!line[0] || strcmpi(line, "rd") == 0)
		printf("Recursion:   %s\n", (rd == 1) ? "true" : "false");
	if (!line[0] || strcmpi(line, "use_tcp") == 0)
		printf("Use TCP:     %s\n", (use_tcp == 1) ? "true" : "false");
	if (!line[0] || strcmpi(line, "timeout") == 0)
		printf("Timeout:     %d\n", timeout);
	return 1;
}

int cmd_help(char *line) {
	int x, p = 0;
	
	for (x = 0; x < n_commands; x++) {
		if (!line[0] ||
			strcmpi(line, commands[x].command) == 0) {
			printf("%-16s %s\n", commands[x].command, commands[x].help);
			p = 1;
		}
	}
	if (p == 0) {
		printf("%s is not a known command!\n", line);
		return -1;
	}
	return 1;
}

int cmd_exit(char *line) {
	return 0;
}

char *no_completion(const char *word, int val) {
	return NULL;
}

char *command_completion(const char *word, int val) {
	static int cur;
	
	if (val == 0)  cur = 0;
	
 	while (cur < n_commands) {
 		if (strncmpi(word, commands[cur].command, strlen(word)) == 0)
 			return strdup(commands[cur++].command);
		cur++;
 	}
	
	return NULL;
}

char *setting_completion(const char *word, int val) {
	static int cur;
	
	if (val == 0) cur = 0;
		
	while (cur < n_settings) {
		if (strncmpi(word, settings[cur], strlen(word)) == 0)
			return strdup(settings[cur++]);
		cur++;
	}

	return NULL;
}

char *bool_completion(const char *word, int val) {
	static int cur = 0;

	if (val == 0) cur = 0;

	if (cur == 0) {
		cur++;
		if (strncmpi(word, "true", strlen(word)) == 0) {
			return strdup("true");
		}
	}

	if (cur == 1) {
		cur++;
		if (strncmpi(word, "false", strlen(word)) == 0) {
			return strdup("false");
		}
	}

	return NULL;
}

#ifdef READLINE_INCLUDED

char **posask_attempted_completion(const char *val, int start, int end) {
	int t, x;

	if (start == 0) return completion_matches((char *)val, (CPFunction *)command_completion);

	/* command, so look which one */
	for (t = 0; t < n_commands; t++) {
		x = strlen(commands[t].command);
		if (strncmpi(rl_line_buffer, commands[t].command, x) == 0 &&
			rl_line_buffer[x] == ' ') {
			return completion_matches((char *)val, (CPFunction *)commands[t].complete);
		}
	}

	return completion_matches((char *)val, (CPFunction *)no_completion);
}

#endif

int clean(int val) {
	closesocklib();
	return val;
}

int main(int argc, char **argv) {
	int x, ret = 0;
	char *ptr;

	/* set default */
	initsocklib();
    getaddress(&server, "127.0.0.1");
	
	/* at first, read the command line */
	for (x = 1; x < argc; x++) {
		if (stricmp(argv[x], "--help") == 0 || stricmp(argv[x], "-h") == 0) {
			printf(
				"Posask - Command-line DNS querier\n"
				"Usage:\n"
				"  posask [--help|-h]\n"
				"  posask [--version|-v]\n"
				"  posask [@[server][:port]] [option=value] [qname] [qtype]\n"
				"If no server is given, 127.0.0.1 is assumed. If no qtype is given, A is\n"
				"assumed. If no qname is given, the program will start in interactive mode.\n"
				"\n"
				"Possible options:\n"
				"  rd=[yes|no]        Specifies whether to demand recursion. Default: yes\n"
                "  serial=n           Default serial number for IXFR requests\n"
                                "  use_tcp=[yes|no]   Specifies whether or not to use TCP. Default: only xfr\n"
				"  timeout=n          Number of seconds to wait for an answer. Default: 5\n"
				"\n"
				"Posask is part of the Posadis DNS Server package. It is distributed under the\n"
				"terms and conditions of the GNU General Public License. More info:\n"
				"   http://posadis.sourceforge.net/\n");
			return clean(1);
		}
		if (stricmp(argv[x], "--version") == 0 || stricmp(argv[x], "-v") == 0) {
			printf(
				"Posask - Command-line DNS querier\n"
				"Version ID:      $Revision: 1.39 $\n"
#ifdef __DATE
				"Compiled on:    " __DATE__
#endif
				"Posadis version: " VERSION "\n");
			return clean(1);
		}
		if (argv[x][0] == '@') {
			/* server */
			if (cmd_server(&argv[x][1]) < 0) return clean(2);
			continue;
		}
		ptr = strchr(argv[x], '=');
		if (ptr) {
			*ptr = '\0';
			ret = issue_command(argv[x], ptr + 1);
			*ptr = '=';
			if (ret < 0) return clean(2);
		} else {
			if (dname_given != 0) {
				/* qtype */
				if (qtype) {
					printf("Didn't expect another command-line argument: %s\n", argv[x]);
					return clean(2);
				} else {
					if (cmd_qtype(argv[x]) < 0) return clean(2);
				}
			} else {
				/* dname */
				if (cmd_dname(argv[x]) < 0) return clean(2);
			}
		}
	}

	if (qtype == 0) qtype = DNS_TYPE_A;

	if (dname_given) {
		/* we're in non-interactive mode here */
		cmd_query("");
		if (ret < 0) return clean(3); else return clean(0);
	} else {
#ifdef READLINE_INCLUDED
		/* interactive mode */
        char *ptr2, *ptr3;
		rl_readline_name = "posask";
#ifdef HAVE_RL_COMPLETION_ENTRY_FUNCTION
		rl_completion_entry_function = (compentry_fn) no_completion;
#else
		completion_entry_function = no_completion;
#endif
        
#ifdef HAVE_RL_COMPLETION_FUNC_T        
        rl_attempted_completion_function = (rl_completion_func_t *)posask_attempted_completion;
#else
        rl_attempted_completion_function = (CPPFunction *)posask_attempted_completion;
#endif

		while (1) {
			ptr = readline(">");
			if (ptr) {
				if (ptr[0]) {
					add_history(ptr);
					ptr2 = strchr(ptr, ' ');
					if (ptr2 != NULL) {
						*ptr2++ = '\0';
						/* trim last space */
						ptr3 = &ptr2[strlen(ptr2) - 1];
						if (*ptr3 == ' ') *ptr3 = '\0';
					} else
						ptr2 = "";
					ret = issue_command(ptr, ptr2);
					if (ret == 0) break;
				}
				free(ptr);
			}
		}
#else
		printf("Sorry, interactive mode not available. GNU Readline was not detected.\n");
#endif
	}

	return clean(0);
}
